/***********************************************************/
/* Camera Thread                                           */
/* The most important class of the application             */
/* This thread controls the capture process                */
/***********************************************************/

#ifndef CAMERA_THREAD_H
#define CAMERA_THREAD_H

/** \file */

#include <QThread>
#include <stdio.h>
#include <FCam/N900.h>

class OverlayWidget;
class InfoWidget;

class CameraThread : public QThread {
    Q_OBJECT;

  public:
    CameraThread(OverlayWidget *o,  char* path, QObject *parent = NULL) : QThread(parent) {
        overlay = o;
        keepGoing = true;
      //  frame = &im;
        this->path = path;
    }
        
  public slots:
    void stop() {
        printf("Stopping!\n");
        keepGoing = false;
    }

signals:
    void startCapturing();
    void finishCapturing();
    void curSave(double);
    void finishSave();
    
  protected:
    void run();

  private:

    //performs the autoExposure without changing the gain
    void autoExpose(FCam::Shot *s, const FCam::Frame &f,
                    int maxExposure = 125000, // 8 fps
                    float smoothness  = 0.5);

    bool keepGoing;
    char *path;
    OverlayWidget *overlay;
   // FCam::Image *frame;
};

#endif
