/***********************************************************/
/* IMPA - VISGRAF                                          */
/* Tássio Knop ( tknop (at) impa (dot) br )                */
/* Alexandre Chapiro ( achapiro (at) impa (dot) br         */
/*                                                         */
/* HDR Capture                                             */
/*                                                         */
/* This app captures frames with varying exposures         */
/* for HDR video                                           */
/***********************************************************/

#include <QApplication>
#include <QPushButton>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QSlider>
#include <QtGui>


#include <QObject>
#include <QMessageBox>
#include "OverlayWidget.h"
#include "CameraThread.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>


int main(int argc, char **argv) {
    QApplication app(argc, argv);

    // Make the main window and a layout for it
    QWidget *window = new QWidget;
    QHBoxLayout *layout = new QHBoxLayout;
    layout->setContentsMargins(0, 0, 0, 0);


    QLabel *rec = new QLabel("<font color='red'>REC</font>",window);
    QLabel *saving = new QLabel("<font color='red'>SAVE(%)</font>",window);
    QLabel *lcurSave = new QLabel("0   ",window);


    rec->setGeometry(40,240-32,540,64);
    rec->setAutoFillBackground(true);
    rec->setVisible(false);

    saving->setGeometry(40,240-32,540,64);
    saving->setAutoFillBackground(true);
    saving->setVisible(false);

    lcurSave->setGeometry(40,240-32,540,64);
    lcurSave->setAutoFillBackground(true);
    lcurSave->setVisible(false);

    // Make an overlay for displaying viewfinder frames
    OverlayWidget *overlay = new OverlayWidget(window);
    overlay->setFixedSize(640, 480);
    layout->addWidget(overlay);

    //File to control the current save. Used to control the number of the captured movie
    //and save it on the appropriate folder
    FILE* nsave = fopen("/home/user/MyDocs/nsave.conf","rt");
    int nsaves = 0;
    if (nsave != NULL){
        fscanf(nsave,"%d",&nsaves);
        fclose(nsave);
    }

    char dirName[550];
    sprintf(dirName,"/home/user/MyDocs/capture%d",nsaves+1);
    //creates a folder which will contain the current capture
    mkdir(dirName, S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);

    nsave = fopen("/home/user/MyDocs/nsave.conf","wt");
    fprintf(nsave,"%d",nsaves+1);
    fclose(nsave);

    // Make some buttons down the right
    QVBoxLayout *buttonLayout = new QVBoxLayout;
    layout->addLayout(buttonLayout);
    QPushButton *quitButton = new QPushButton("X");

    rec->setFixedSize(80, 64);
    saving->setFixedSize(80, 64);
    lcurSave->setFixedSize(80, 64);

    buttonLayout->addWidget(quitButton);
    buttonLayout->addStretch(1);
    buttonLayout->addWidget(rec);
    buttonLayout->addWidget(saving);
    buttonLayout->addWidget(lcurSave);


    window->setLayout(layout);

    // Make a thread that controls the camera
    CameraThread cameraThread(overlay, dirName);

    // Hook up the quit button to stop the camera thread
    QObject::connect(quitButton, SIGNAL(clicked()), 
                     &cameraThread, SLOT(stop()));

    // Once the camera thread stops, quit the app
    QObject::connect(&cameraThread, SIGNAL(finished()), 
                     &app, SLOT(quit()));

    QObject::connect(&cameraThread, SIGNAL(startCapturing()),
                     rec, SLOT(show()));
    QObject::connect(&cameraThread, SIGNAL(startCapturing()),
                     lcurSave, SLOT(hide()));
    QObject::connect(&cameraThread, SIGNAL(startCapturing()),
                     saving, SLOT(hide()));
    QObject::connect(&cameraThread, SIGNAL(finishCapturing()),
                     rec, SLOT(hide()));
    QObject::connect(&cameraThread, SIGNAL(finishCapturing()),
                     saving, SLOT(show()));
    QObject::connect(&cameraThread, SIGNAL(finishCapturing()),
                     lcurSave, SLOT(show()));
    QObject::connect(&cameraThread, SIGNAL(finishSave()),
                     lcurSave, SLOT(hide()));
    QObject::connect(&cameraThread, SIGNAL(finishSave()),
                     saving, SLOT(hide()));
    QObject::connect(&cameraThread, SIGNAL(curSave(double)),
                     lcurSave, SLOT(setNum(double)));

    // Show the app full screen
    window->showFullScreen();

    // Launch the camera thread
    cameraThread.start();

    // Enter the QT main event loop
    return app.exec();
}
